/* globals ExtensionAPI */

Cu.importGlobalProperties(["fetch"]);

ChromeUtils.defineModuleGetter(this, "Services",
  "resource://gre/modules/Services.jsm");
ChromeUtils.defineModuleGetter(this, "ExtensionCommon",
  "resource://gre/modules/ExtensionCommon.jsm");
ChromeUtils.defineModuleGetter(this, "AddonManager",
  "resource://gre/modules/AddonManager.jsm");
ChromeUtils.defineModuleGetter(this, "Preferences",
  "resource://gre/modules/Preferences.jsm");
ChromeUtils.defineModuleGetter(this, "AddonStudies",
  "resource://normandy/lib/AddonStudies.jsm");

let gExtension;

this.multipreffer = class extends ExtensionAPI {
  getAPI(context) {
    gExtension = context.extension;
    FirefoxHooks.init();

    return {
      multipreffer: {
        async studyReady(studyInfo) {
          await FirefoxHooks.studyReady(studyInfo);
        },
      },
    };
  }
};

this.FirefoxHooks = {
  init() {
    AddonStudies.addUnenrollListener(gExtension.id, async (reason) => {
      await this.cleanup();
    });
    AddonManager.addAddonListener(this);
  },

  get abortedPref() {
    return `extensions.multipreffer.${gExtension.id}.aborted`;
  },

  async studyReady(studyInfo) {
    // Called every time the add-on is loaded.
    this.studyInfo = studyInfo;
    if (Preferences.get(this.abortedPref)) {
      return;
    }

    try {
      const res = await fetch(gExtension.getURL("variations.json"));
      this.variations = await res.json();
    } catch (e) {
      console.error("Failed to load variations!");
      return;
    }

    const variationName =
      Preferences.get("extensions.multipreffer.test.variationName",
                      this.studyInfo.variation.name);
    const prefs = this.variations[variationName].prefs;
    if (this.studyInfo.isFirstRun) {
      for (const name of Object.keys(prefs.setValues)) {
        if (!prefs.expectNonDefaults.includes(name) && Preferences.isSet(name)) {
          // One of the prefs has a user-set value, ABORT!!!
          // TODO: End the study/uninstall the addon?
          Preferences.set(this.abortedPref, true);
          return;
        }
      }
    }
    Preferences.set(prefs.setValues);
  },

  async cleanup() {
    // Only run once. cleanup may get called from multiple listeners on expiry.
    if (this._cleanedUp) {
      return;
    }
    // Called when the add-on is being removed for any reason.
    if (Preferences.get(this.abortedPref)) {
      Preferences.reset(this.abortedPref);
      return;
    }

    try {
      const variationName = Preferences.get("extensions.multipreffer.test.variationName", this.studyInfo.variation.name);
      const prefs = this.variations[variationName].prefs;
      const setValues = prefs.setValues;

      // Handle the prefs that need to be reset to default.
      const resetDefaults = [];
      for (const pref of prefs.resetDefaults) {
        if (Preferences.get(pref) === setValues[pref]) {
          // Pref value hasn't changed from what we set. Include it.
          resetDefaults.push(pref);
        }
      }
      Preferences.reset(resetDefaults);

      // Handle the prefs that need to be set to a specified value.
      const resetValues = prefs.resetValues;
      for (const [name, value] of Object.entries(setValues)) {
        if (Preferences.get(name) !== value) {
          // Pref was modified. Make sure it's not in the resetValues list.
          delete resetValues[name];
        }
      }
      Preferences.set(resetValues);

      // At this point, the category pref has been wiped; let's recompute it.
      // BrowserGlue will recompute the category pref when the cookie behavior
      // pref changes value, but only if the category pref is unset. So, we
      // need to change the cookie behavior pref, which will recompute the
      // category pref. Then, clear the newly recomputed category pref, and
      // finally restore the cookie behavior pref again which will recompute
      // the category pref in the correct state.
      const cookieBehaviorValue =
        Preferences.get("network.cookie.cookieBehavior");
      const tempCookieBehaviorValue = cookieBehaviorValue === 4 ? 0 : 4;
      Preferences.set("network.cookie.cookieBehavior", tempCookieBehaviorValue);
      Preferences.reset("browser.contentblocking.category");
      Preferences.set("network.cookie.cookieBehavior", cookieBehaviorValue);
      this._cleanedUp = true;
    } catch (e) {
      Cu.reportError(e);
    }
  },

  onUninstalling(addon) {
    this.handleDisableOrUninstall(addon);
  },

  onDisabled(addon) {
    this.handleDisableOrUninstall(addon);
  },

  async handleDisableOrUninstall(addon) {
    if (addon.id !== gExtension.id) {
      return;
    }
    await this.cleanup();
    AddonManager.removeAddonListener(this);
    // This is needed even for onUninstalling, because it nukes the addon
    // from UI. If we don't do this, the user has a chance to "undo".
    addon.uninstall();
  },
};
